    //
//  NumberPickerController.m
//  FamigoiPhoneExample
//
//  Created by Cody Powell on 9/20/10.
//  Copyright 2010 Famigo. All rights reserved.
//

#import "NumberPickerController.h"
#import "NumberPickerConstants.h"
#import "FamigoHeaders.h"
/*#import "ApiDataAccess.h" // Defines all of the constants across the SDK.
#import "FamigoController.h" // Main UI entry point for SDK.
#import "Famigo.h" // Singleton storing all family and member information.
#import "AsyncTurnView.h" // Shows whose turn it is.
#import "GearTab.h" // Allows user to cancel and go back to Famigo screens.*/

#define G_SUBVIEW_FRAME CGRectMake(0, 20, 320, 460)
#define MAX_NUMBER 100

#define INTEGRATE_FAMIGO_TAG 0
#define FINISHED_GAME_TAG 1
#define CANCEL_TAG 2

#define BACK_TO_FAMIGO_INDEX 0
#define CANCEL_INDEX 1
#define HELP_INDEX 2

@interface NumberPickerController()
-(void)promptForFamigo;
-(BOOL)isIntegratedWithFamigo;
-(void)launchFamigoController;
-(void)generateRandomNumberAndUpdateGame;
-(void)updateViewForGameUpdate;
-(void)drawTurnView;
-(void)displayFinishedGame;
-(void)drawGearTab;
-(void)cancelGame;
-(void)launchHelpController;
@end

@implementation NumberPickerController

@synthesize famigoController, turnView, gearTab;

/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	[self drawGearTab];
	
	// Is the user interested in integrating with Famigo?  Why wouldn't they be!
	[self promptForFamigo];
}

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)dealloc {
    [super dealloc];
}

#pragma mark Famigo Integration
- (void)promptForFamigo {
	// Is this the first time this game has been started?  If so, ask the user if they want to integrate with Famigo.
	NSUserDefaults *db = [NSUserDefaults standardUserDefaults];
	NSString *introMessage = [db stringForKey:FC_d_intro_message_displayed];
	
	// Alert the user about Famigo only once - we don't want to annoy.
	if (introMessage == nil) {
		UIAlertView *av = [[UIAlertView alloc] initWithTitle:@"Want to integrate with Famigo?" 
													 message:@"Famigo lets you connect with family and friends through games, so you can challenge each other, share scores, chat, and much more!\n\nDo you want to integrate this game with Famigo?"
													delegate:self 
										   cancelButtonTitle:@"No thanks"
										   otherButtonTitles:@"Absolutely!",nil];
		av.tag = INTEGRATE_FAMIGO_TAG;
		[av show];
		[av autorelease];
		
		// set the value so this doesn't happen anymore.
		[db setValue:@"YES" forKey:FC_d_intro_message_displayed];
	}
	else {
		// If the user has already seen this popup, skip it and determine whether or not we go into the Famigo screens.
		if ([self isIntegratedWithFamigo])
			[self launchFamigoController];
	}
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {	
	if (alertView.tag == INTEGRATE_FAMIGO_TAG) {
		NSUserDefaults *db = [NSUserDefaults standardUserDefaults];
		switch (buttonIndex) {
			case 0: // Don't wanna integrate with Famigo.
				[db setValue:@"NO" forKey:FC_d_famigo_enabled];
				[db	synchronize];
				break;
			case 1: // Let's totally integrate, dude.
				[db setValue:@"YES" forKey:FC_d_famigo_enabled];
				[db synchronize];
				[self launchFamigoController];
				break;
			default:
				NSAssert(0, @"Oops, we have no idea which button you clicked.");
				break;
		}
	}
	
	if (alertView.tag == CANCEL_TAG) {
		if (buttonIndex == alertView.cancelButtonIndex) {
			Famigo *f = [Famigo sharedInstance];
			[f cancelGame];
			[self launchFamigoController];
		}
	}
}

-(BOOL)isIntegratedWithFamigo {
	NSUserDefaults *db = [NSUserDefaults standardUserDefaults];
	// If the user has selected not to enable Famigo, we're all done here.
	NSString *famigoEnabled = [db stringForKey:FC_d_famigo_enabled];
	if (![famigoEnabled isEqualToString:@"YES"])
		return NO;
	
	// The Famigo singleton will send us notes related to important game happenings, so we better be listening.
	[[Famigo sharedInstance] registerForNotifications:self withSelector:@selector(famigoNote:)];
	return YES;
}

-(void)launchFamigoController {	
	// Throw the Famigo controller up on screen.  Once the user is done in there, Famigo will call the famigoReady callback
	// and we can continue with gameflow.
	if (!self.famigoController) {
		self.famigoController = [[FamigoController sharedInstanceWithDelegate:self] autorelease];
		[famigoController.view setAutoresizingMask:(UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight)];
		famigoController.view.frame = G_SUBVIEW_FRAME;
	}
	
	[self.view addSubview:famigoController.view];
    [famigoController viewWillAppear:NO];
	[self.famigoController show];
}
 
#pragma mark Famigo Controller Delegate
-(void)famigoReady {
	// This is called when the FamigoController disappears.  We actually don't need to do anything
	// when the FamigoController disappears, as we wait for a famigo note.
}

#pragma mark Famigo Game Messages
- (void)famigoNote:(NSNotification*)note {
	NSString *noteName = [note name];
	NSLog(@"received note %@", noteName);
	
	if ([noteName isEqualToString:FamigoMessageGameReady]) {
		// We receive the game ready note when the game data has been updated via the API and is ready
		// to be processed in the game.  Note this only fires when game data has changed.
		[self updateViewForGameUpdate];
		
	} else if ([noteName isEqualToString:FamigoMessageGameCreated]) {
		// Hooray, a new game has been created!  Now let's pick the number that we'll try to guess.
		[self generateRandomNumberAndUpdateGame];
		
	} else if ([noteName isEqualToString:FamigoMessageGameCanceled]) {
		// Boooooo, somebody canceled the game!  Now would be a good time to start crying.
		
	} else if ([noteName isEqualToString:FamigoMessageGameFinished]) {
		// Somebody won!  I hope it was me.
		[self displayFinishedGame];
	}
}	

#pragma mark Famigo Utility Methods
-(NSDictionary*)getNextPlayer {
	Famigo *f = [Famigo sharedInstance];
	NSArray *players = [f.gameInstance valueForKey:FC_d_game_players];
	NSString *currentPlayerMemberId = [f.gameInstance valueForKey:FC_d_game_current_turn];
	
	// Must find the member AFTER the current member in the players array.  They get next turn.
	BOOL currentPlayerHasBeenFound = NO;
	for (NSDictionary *playerDict in players) { 
		NSString *memberId = [playerDict valueForKey:FC_d_member_id];
		if ([memberId isEqualToString:currentPlayerMemberId]) {
			currentPlayerHasBeenFound = YES;
			continue;
		}
		else {
			if (currentPlayerHasBeenFound)
				return playerDict;
		}
	}
	
	// If we made it to the end of the array without finding the next player, the next player
	// is the first member in the array.
	NSDictionary *firstPlayer = [players objectAtIndex:0];
	return firstPlayer;
}

#pragma mark Number Picker UI
-(void) updateViewForGameUpdate {
	// Only allow a player to guess if it's currently his turn.
	Famigo *f = [Famigo sharedInstance];
	NSString *currentTurnMember = [f.gameInstance valueForKey:FC_d_game_current_turn];
	submitGuess.enabled = [currentTurnMember isEqualToString:f.member_id];
	
	// We want to show whose turn it is all through the game. We've got a nice control for that.
	// If the turnview is already on the screen, let's throw it away and redraw it to reflect 
	// the change in player ordering.
	if (self.turnView) {
		[self.turnView removeFromSuperview];
		[self.turnView release];
		self.turnView = nil;
	}
	[self drawTurnView];
}

-(void) drawTurnView {
	// You can specify any color to spice this control up.
	UIColor *blue = [UIColor colorWithRed:0/255.0 green:160/255.0 blue:210/255.0 alpha:1.0];
	
	// Place this at the bottom of the view.
	int turnViewHeight = 110;
	int turnViewBottomBuffer = 50;
	int turnViewYCoord = self.view.bounds.size.height - turnViewHeight - turnViewBottomBuffer;
	int turnViewWidth = self.view.bounds.size.width;
	int turnViewXCoord = 0;
	CGRect turnViewRect = CGRectMake(turnViewXCoord, turnViewYCoord, turnViewWidth, turnViewHeight);
	// If we wanted to specify a different color for each player in the turnview, we could by passing an array of
	// UIColors to textColors. We don't care about that here though.
	self.turnView = [[AsyncTurnView alloc] initWithFrameAndColor:turnViewRect shapeColor:blue showScores:YES textColors:nil];
	[self.view addSubview:turnView];
}

-(void)drawGearTab {
	int gearTabCenterX = 310;
	int gearTabCenterY = 440;
	GearTab *gt = [[GearTab alloc] initWithFrame:CGRectZero];
	gt.center = CGPointMake(gearTabCenterX, gearTabCenterY);
	[gt setAutoresizingMask:(UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleTopMargin)];
	[gt addTarget:self action:@selector(gearAction:) forControlEvents:UIControlEventTouchUpInside];
	[self.view addSubview:gt];
	[gt release];
	self.gearTab = gt;
}

#pragma mark Number Guessing Stuff
-(void)sliderChanged:(id)sender {
	int roundedSliderValue = round(slider.value);
	sliderLabel.text = [NSString stringWithFormat:@"%d", roundedSliderValue];
}

-(void)submitGuessClicked:(id)sender {
	// Did the user just guess the magic number?
	// This value is persisted in our game dictionary, under the key specified in the
	// generateRandomNumberAndUpdateGame method.
	Famigo *f = [Famigo sharedInstance];
	NSDictionary *gameDictionary = [f.gameInstance valueForKey:f.game_name];
	NSNumber *numberToGuess = [gameDictionary valueForKey:FC_number_to_guess];
	NSNumber *guess = [NSNumber numberWithInt:round(slider.value)];
	
	// Let's update the list of guesses regardless of whether we've guessed correct or if the 
	// game keeps going.
	NSMutableArray *numberGuesses = [gameDictionary valueForKey:FC_number_guesses];
	[numberGuesses addObject:guess];
	[gameDictionary setValue:numberGuesses forKey:FC_number_guesses];
	
	// Hooray, you win!  Well, you did if you guessed right.
	if ([numberToGuess intValue] == [guess intValue]) {
		// Setting a game's active property to finished will generate a FamigoMessageGameFinished
		// note.
		[f.gameInstance setValue:@"finished" forKey:FC_d_game_active];
		
		// Also, the winning player gets a score of 1 point.  Woohoo for them!
		NSString *winningPlayerMemberId = f.member_id;
		NSArray *players = [f.gameInstance valueForKey:FC_d_game_players];
		
		for (NSDictionary *player in players) {
			NSString *playerMemberId = [player valueForKey:FC_d_member_id];
			if ([playerMemberId isEqualToString:winningPlayerMemberId]) {
				[player setValue:@"1" forKey:FC_d_score];
			}
		}
	}
	else {
		// You didn't guess correctly, so the turn passes to the next person.  
		// Persist that in our game instance.
		NSDictionary *nextPlayer = [self getNextPlayer];
		NSString *nextPlayerId = [nextPlayer valueForKey:FC_d_member_id];
		[f.gameInstance setValue:nextPlayerId forKey:FC_d_game_current_turn];
		
		// We also give a little hint so your guesses can gain precision.
		if (guess > numberToGuess)
			guessHint.text = @"Too high!";
		else 
			guessHint.text = @"Too low!";
	}
	
	[f updateGame];
}

#pragma mark Number Picker Gameplay
-(void)generateRandomNumberAndUpdateGame {
	// The game instance contains everything about the game. It contains the players, when we started the game,
	// the game id, game status, and so forth.  Most of that is used inside the API, not by us.
	Famigo *f = [Famigo sharedInstance];
	NSDictionary *gameInstance = f.gameInstance;
	
	// When a game is created, we need to set some game-specific information in there, like the number we're
	// trying to guess and our previous guesses.  We persist the game-specific info under a key matching the 
	// game's name.  That's how we know that data is safe for us to touch.
	NSNumber *numberToGuess = [NSNumber numberWithInt:arc4random() % MAX_NUMBER];
	NSMutableArray *numberGuesses = [[[NSMutableArray alloc] init] autorelease];
	NSDictionary *newGameDictionary = [NSDictionary dictionaryWithObjectsAndKeys:numberToGuess, FC_number_to_guess,
									   numberGuesses, FC_number_guesses, nil];
	[gameInstance setValue:newGameDictionary forKey:f.game_name];
	
	// Let's also let the game know that I'm the current player.  That's how the game switcher shows the user
	// when it's their turn versus somebody else's turn.
	[gameInstance setValue:f.member_id forKey:FC_d_game_current_turn];
	
	// Push the changes to the API.  This should result in an immediate GameReady message.
	[f updateGame];
	
	NSLog(@"Pushing game dict of %@", gameInstance);
}

-(void)displayFinishedGame {
	submitGuess.enabled = NO;
	
	// Somebody won the game.  The winner will be whoever played last, but that only gives us the member id.
	Famigo *f = [Famigo sharedInstance];
	NSString *currentTurnMemberId = [f.gameInstance valueForKey:FC_d_game_current_turn];
	NSString *winningPlayerName = @"";
	
	// Iterate through our list of players, finding the name of the player who won.
	NSArray *players = [f.gameInstance valueForKey:FC_d_game_players];
	for (NSDictionary *playerDictionary in players) {
		NSString *memberId = [playerDictionary valueForKey:FC_d_member_id];
		NSString *memberName = [playerDictionary valueForKey:FC_d_member_name];
		
		if ([currentTurnMemberId isEqualToString:memberId]) {
			winningPlayerName = memberName;
			break;
		}
	}
	
	if (self.turnView) {
		[self.turnView removeFromSuperview];
		[self.turnView release];
		self.turnView = nil;
	}
	[self drawTurnView];
	
	UIAlertView *av = [[UIAlertView alloc] initWithTitle:@"Game over!" 
												 message:[NSString stringWithFormat:@"%@ wins!", winningPlayerName]
												delegate:self 
									   cancelButtonTitle:@"OK"
									   otherButtonTitles:nil];
	av.tag = FINISHED_GAME_TAG;
	[av show];
	[av autorelease];
}

#pragma mark Gear Tab Whatnot
- (void)gearAction:(id)sender {
	UIActionSheet *as = [[UIActionSheet alloc] initWithTitle:nil
													delegate:self 
										   cancelButtonTitle:@"Continue Playing" 
									  destructiveButtonTitle:nil 
										   otherButtonTitles:@"Back to Famigo",@"Cancel Game",@"Help",nil];
	[as showInView:self.view];
	[as release];	
}

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex {
	NSInteger cancelIndex = [actionSheet cancelButtonIndex];
	
	if (buttonIndex != cancelIndex) {
		if (buttonIndex == BACK_TO_FAMIGO_INDEX) {
			[self launchFamigoController];
		}
		else if (buttonIndex == CANCEL_INDEX) {
			[self cancelGame];
		} 
		else if (buttonIndex == HELP_INDEX) {
			[self launchHelpController];
		}
	}
}

- (void)cancelGame { 
	UIAlertView *av = [[UIAlertView alloc] initWithTitle:@"Cancel Game?"
												 message:@"Are you sure? This action can NOT be undone." 
												delegate:self 
									   cancelButtonTitle:@"Cancel Game" 
									   otherButtonTitles:@"Keep Playing",nil];
	av.tag = CANCEL_TAG;
	[av show];
	[av release];
}

- (void) launchHelpController {
	[self launchFamigoController];
	[self.famigoController launchHelpController];
}
@end
